/*
 *  TU Eindhoven
 *  Eindhoven, The Netherlands
 *
 *  Name            :   cluster.h
 *
 *  Author          :   Sander Stuijk (sander@ics.ele.tue.nl)
 *
 *  Date            :   July 19, 2005
 *
 *  Function        :   SDF actor cluster
 *
 *  History         :
 *      19-07-05    :   Initial version.
 *
 * $Id: cluster.cc,v 1.1.1.1 2007/10/02 10:59:46 sander Exp $
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * In other words, you are welcome to use, share and improve this program.
 * You are forbidden to forbid anyone else to use, share and improve
 * what you give them.   Happy coding!
 */

#include "cluster.h"

/**
 * SDFactorCluster ()
 * Constructor.
 */
SDFactorCluster::SDFactorCluster(SDFcomponent &c)
    :
        SDFcomponent(c)
{
}

/**
 * ~SDFactorCluster ()
 * Destructor.
 */
SDFactorCluster::~SDFactorCluster()
{
}

/**
 * getActor ()
 * The function returns a reference to an actor with the given name or NULL
 * if the actor is not part of the cluster.
 */
SDFactor *SDFactorCluster::getActor(const CString &name)
{
    for (SDFactorsIter iter = actors.begin(); iter != actors.end(); iter++)
    {
        SDFactor *a = *iter;
        
        if (a->getName() == name)
            return a;
    }
    
    return NULL;
}

/**
 * addActor ()
 * Add an actor to the cluster.
 */
void SDFactorCluster::addActor(SDFactor *a)
{
    actors.push_back(a);
}

/**
 * removeActor ()
 * Remove an actor from the cluster.
 */
void SDFactorCluster::removeActor(const CString &name)
{
    for (SDFactorsIter iter = actors.begin(); iter != actors.end(); iter++)
    {
        SDFactor *a = *iter;
        
        if (a->getName() == name)
        {
            actors.erase(iter);
            return;
        }
    }

    throw CException("Cluster '" + getName() 
                        + "' has no actor '" + name + "'.");
}

/**
 * print ()
 * Print the cluster to the supplied output stream.
 */
ostream &SDFactorCluster::print(ostream &out)
{
    out << "Cluster (" << getName() << ")" << endl;
    out << "id:        " << getId() << endl;
    out << "actors:    ";

    for (SDFactorsIter iter = actors.begin(); iter != actors.end(); iter++)
    {
        SDFactor *a = *iter;
        
        out << a->getName() << " ";
    }
    out << endl;
    out << endl;
        
    return out;
}
