/*
 *  TU Eindhoven
 *  Eindhoven, The Netherlands
 *
 *  Name            :   scheduler.h
 *
 *  Author          :   Sander Stuijk (sander@ics.ele.tue.nl)
 *
 *  Date            :   July 27, 2005
 *
 *  Function        :   Scheduler for SDF graphs
 *
 *  History         :
 *      27-07-05    :   Initial version.
 *
 * $Id: scheduler.h,v 1.1.1.1 2007/10/02 10:59:47 sander Exp $
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * In other words, you are welcome to use, share and improve this program.
 * You are forbidden to forbid anyone else to use, share and improve
 * what you give them.   Happy coding!
 */

#ifndef SDF_RESOURCE_ALLOCATION_SCHEDULING_SCHEDULER_H_INCLUDED
#define SDF_RESOURCE_ALLOCATION_SCHEDULING_SCHEDULER_H_INCLUDED

#include "../../base/timed/graph.h"
#include "../mpsoc_arch/graph.h"

/**
 * stateSpaceStaticOrderSchedule ()
 * The function constructs a set of static-order schedules for the processors
 * in the architecture platform. Each schedule orders the firings of the actors
 * running on the processor.
 */
void stateSpaceStaticOrderSchedule(TimedSDFgraph *gr, PlatformGraph *ap,
        unsigned long long stackSz = 1000, unsigned long long hashSz = 10000);

/**
 * stateSpacePriorityStaticOrderSchedule ()
 * The function constructs a set of static-order schedules for the processors
 * in the architecture platform. Each schedule orders the firings of the actors
 * running on the processor.
 */
void stateSpacePriorityStaticOrderSchedule(TimedSDFgraph *gr, PlatformGraph *ap,
        unsigned long long stackSz = 1000, unsigned long long hashSz = 10000);

/**
 * List scheduler
 * Constructs a set of static-order schedules for the processors in the
 * architecture platform. Each schedule orders the firings of the actors
 * running on the processor. A list scheduler is used to construct the
 * static-order schedules.
 */
#include "list_scheduler.h"

/**
 * latencyAnalysisForRandomStaticOrderSingleProc ()
 * The function computes the latency for a random static-order schedule on
 * a single processor system. The minimal latency for the specified number of
 * attempts is returned.
 */
SDFtime latencyAnalysisForRandomStaticOrderSingleProc(TimedSDFgraph *g,
        SDFactor *srcActor, SDFactor *dstActor, uint nrAttempts);

class scheduler
{
public:
    // Scheduler status values
    enum {SCHEDULER_OK = 1, SCHEDULER_ERROR = 0};

    // Constructor
    scheduler() {};
    
    // Destructor
    virtual ~scheduler() {};
    
    // Run the scheduler on the graph g
    virtual int run(TimedSDFgraph *g) = 0;
    
    // Check whether all preconditions are satisfied
    virtual int check(TimedSDFgraph *g) = 0;
    
    // Reset the scheduler
    virtual void reset() = 0;
};

#endif
