/*
 *  TU Eindhoven
 *  Eindhoven, The Netherlands
 *
 *  Name            :   sp_earliestdeadline.h
 *
 *  Author          :   Sander Stuijk (sander@ics.ele.tue.nl)
 *
 *  Date            :   July 27, 2005
 *
 *  Function        :   Earliest Deadline First scheduler for single processor
 *
 *  History         :
 *      27-07-05    :   Initial version.
 *
 * $Id: sp_earliestdeadline.h,v 1.1.1.1 2007/10/02 10:59:47 sander Exp $
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * In other words, you are welcome to use, share and improve this program.
 * You are forbidden to forbid anyone else to use, share and improve
 * what you give them.   Happy coding!
 */

#ifndef SDF_RESOURCE_ALLOCATION_SCHEDULING_SP_EARLIESTDEADLINE_H_INCLUDED
#define SDF_RESOURCE_ALLOCATION_SCHEDULING_SP_EARLIESTDEADLINE_H_INCLUDED

#include "scheduler.h"
#include "schedule.h"
#include "../../transform/to_apg/apg.h"

class SPearliestDeadlineScheduler : public scheduler
{
public:
    // Constructor
    SPearliestDeadlineScheduler();
    
    // Destructor
    ~SPearliestDeadlineScheduler();
    
    // Run the scheduler on the graph g
    int run(TimedSDFgraph *g);
    
    // Check whether all preconditions are satisfied
    int check(TimedSDFgraph *g);
    
    // Reset the scheduler
    void reset();

    // Schedule
    SPschedule *getSchedule() { return schedule; };

private:
    // Ready list
    APGnode *getNodeWithEarliestDeadline(APGnodes &readyList);
    void initReadyList(APGnodes &readyList, APGgraph *g);
    void updateReadyList(APGnodes &readyList, APGnode *v, APGgraph *g);
    void computeDeadlines(APGgraph *g);
    int maxWeightOnAllOutEdges(APGnode *n);
    bool positiveWeightOnAllOutEdges(APGnode *n);
    void addNodeToReadyList(APGnodes &readyList, APGnode *n);           
    
    // Schedule
    SPschedule *schedule;
    
    // Graph
    TimedSDFgraph *hsdfGraph;
};

#endif
